/* =============================================================================
 *
 * CapSense_Demo.c
 *
 * This file contains the code for the CapSense demo task.
 *
 * v1.0	YFS	Initial release. 
 *
 * =============================================================================
*/

/*
* Copyright (2014), Cypress Semiconductor Corporation. All Rights Reserved.
*
* This software is owned by Cypress Semiconductor Corporation (Cypress)
* and is protected by and subject to worldwide patent protection (United
* States and foreign), United States copyright laws and international treaty
* provisions. Cypress hereby grants to licensee a personal, non-exclusive,
* non-transferable license to copy, use, modify, create derivative works of,
* and compile the Cypress Source Code and derivative works for the sole
* purpose of creating custom software in support of licensee product to be
* used only in conjunction with a Cypress integrated circuit as specified in
* the applicable agreement. Any reproduction, modification, translation,
* compilation, or representation of this software except as specified above 
* is prohibited without the express written permission of Cypress.
*
* Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH 
* REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
* Cypress reserves the right to make changes without further notice to the 
* materials described herein. Cypress does not assume any liability arising out 
* of the application or use of any product or circuit described herein. Cypress 
* does not authorize its products for use as critical components in life-support 
* systems where a malfunction or failure may reasonably be expected to result in 
* significant injury to the user. The inclusion of Cypress' product in a life-
* support systems application implies that the manufacturer assumes all risk of 
* such use and in doing so indemnifies Cypress against all charges. 
*
* Use of this Software may be limited by and subject to the applicable Cypress
* software license agreement. 
*/

/* Include FreeRTOS APIs and defines */
#include <FreeRTOS.h>
#include <task.h>
#include <queue.h>
#include <event_groups.h>

/* Include PSoC system and component APIs and defines */
#include <project.h>

/* Include application function declarations and defines */
#include <utils.h>
#include <App_Defs.h>
#include <Task_Defs.h>

/*
 * Function:	CapSense_Demo_Task
 *
 * This function runs as a FreeRTOS task with a medium priority. It is a
 * demonstration of how to extract finger touches from the CapSense queue
 * and print the results to the LCD.
 *
 * Execution of this task is controlled by an event group. After creating
 * this task, the main task starts the demo by setting the bit in Demo_Events
 * that corresponds to this demo. When the user signals a desire to end the
 * demo, it flushes the message queue (of touches that are no longer relevant)
 * and then waits on the event bit again. This leaves the task ready to be
 * re-executed at a later date.
 *
 *			Start task
 *		 ->	Check the demo stop flag. If so...
 *		|		Wait on its event bit (sleep) 
 *      |   	Clear LCD and print demo text
 *		|		Clear the stop flag
 *		|	Get data from the CapSense queue (non-blocking)
 *		|   Print results to LCD (if changed)
 *		|   Check if SW3 is pressed. If so...
 *		|   	Flush the CapSense queue
 *		|   	Set the Main_Task event bit (make it ready to run)
 *		|   	Set the stop flag
 *		 -- Loop 
 *
 * Globals:		Uses Demo_Events, Main_Task_Event and CapSense_Monitor_Q
 *
 * Parameters:	pvParameters (demo index number)
 *
 * Return:		None (infinite loop)
 */
 void CapSense_Demo_Task( void *pvParameters )
{
	int demoNumber = (int)pvParameters;     /* Passed from xTaskCreate */
    
    BaseType_t queueDataReceived;           /* Return from xQueueReceive */
	
    CS_PACKET packet;	/* Hold a message from CapSense Q */

	int stop_demo = 1;	/* Flag for SW3 press (and first time into the code) */
	
    for( ; ; )		/* Demo main loop (forever) */
    {
		/* Execute when task first starts and when the demo is started */
		if( stop_demo )
		{
            /* Wait on the demo event bit so that Main_Task can run */
            xEventGroupWaitBits(
                Demo_Events,        /* Wait for this demo to be requested */
                1 << demoNumber,    /* Event to wait - each demo owns a bit */
                pdTRUE,             /* Clear the event on exit */
                pdFALSE,            /* Do not wait for all bits (irrelevant) */
                portMAX_DELAY );    /* Wait forever */
	
			/*
			MainTask has set the event bit so it is time to run the demo.
			Prepare the LCD to show the demo results.
			*/
			LCD_ClearDisplay();
			LCD_PrintString( CAPSENSE_DEMO_TASK );
			LCD_Position( BOTTOM_ROW, ROW_START );
			LCD_PrintString( "Touch something!" );
		
			/* Clear the flag so we only run this once per demo run */
			stop_demo = 0;
		
		} /* if( stop_demo ) */	
		
		/* Collect the CapSense activity from the message queue */
        queueDataReceived = xQueueReceive(
            CapSense_Monitor_Q,
            &packet.result,
            DO_NOT_WAIT_ON_QUEUE );

        if( queueDataReceived == pdTRUE )   /* Call returned usable data */
		{
			/* Clear, then change the display to match what was pressed */
			LCD_Position( BOTTOM_ROW, ROW_START );
			LCD_PrintString( SIXTEENSPACES );
			LCD_Position( BOTTOM_ROW, ROW_START );
			
			LCD_PrintString( packet.b0 ? "B0 " : "   " );
			LCD_PrintString( packet.b1 ? "B1 " : "   " );
			
			if( packet.slider )
				LCD_PrintNumber( packet.sliderval );					
		} /* if( queueDataReceived == pdTRUE ) */

		/* Check to see if SW3 was pressed - this signals the end of the demo */
		if( ! P15_5_Read() )		/* Note: 0 (ground) means SW3 was pressed */
		{	
			/* Flush any CapSense data from queue - it is no longer relvant */
			xQueueReset( CapSense_Monitor_Q );
			
			/* Set the MainTask event bit to make it ready to run */
			xEventGroupSetBits( Main_Task_Event, MAIN_TASK_EVENT_NUM );

			/* Set flag to go to sleep (allow Main_Task to run) */
			stop_demo = 1;
			
		} /* if( ! P15_5_Read() ) */	
		
    } /* for( ; ; ) */
	
} /* CapSense_Demo_Task() */
